asm MorraCinese_raff3

//Morra cinese (Wikipedia)
//Lo scopo del gioco e' sconfiggere l'avversario scegliendo un segno
//in grado di battere quello dell'altro, secondo le seguenti regole:
//1. Il sasso spezza le forbici (vince il sasso)
//2. Le forbici tagliano la carta (vincono le forbici)
//3. La carta avvolge il sasso (vince la carta)
//Se i due giocatori scelgono la stessa arma, il gioco e' pari
//e si gioca di nuovo.

//Questo modello ASM permette di giocare a morra cinese contro il computer.

//E' il terzo raffinamento del modello MorraCinese.asm (raffinamento di
//MorraCinese_raff2.asm): se, dopo aver giocato il numero massimo di partite,
//il computer e l'utente sono in parita', permette che continuino a giocare
//fino a quando uno dei due non vince.

import StandardLibrary

signature:
	enum domain Sign = {CARTA | FORBICE | SASSO}
	enum domain Result = {WINFIRST | WINSECOND | PATTA}
	dynamic monitored userChoice: Sign //scelta dell'utente
	dynamic controlled computerChoice: Sign //scelta del computer

	dynamic controlled userWins: Natural //numero di vittorie dell'utente
	dynamic controlled computerWins: Natural //numero di vittorie del computer

	dynamic controlled numberOfPlays: Natural
	dynamic controlled maxNumberOfPlays: Natural
	dynamic monitored maxNumberOfPlaysInit: Natural

	dynamic controlled outMess: String //stampa a video del risultato della partita
	static playResult: Prod(Sign, Sign) -> Result //data una coppia di giocate, ritorna il risultato

definitions:

	//La funzione e' completamente specificata.
	//Infatti, tutte le 9 combinazioni sono considerate.
	function playResult($s1 in Sign, $s2 in Sign) =
		if($s1=$s2) then //tre casi
			PATTA
		else
			switch($s1, $s2)
				case (FORBICE, CARTA):
					WINFIRST
				case (CARTA, SASSO):
					WINFIRST
				case (SASSO, FORBICE):
					WINFIRST
				otherwise //i tre casi rimanenti
					WINSECOND
			endswitch
		endif

	main rule r_Main =
		// a or (not(a) and b)
		if((numberOfPlays < maxNumberOfPlays) or
			(numberOfPlays >= maxNumberOfPlays and userWins = computerWins)) then
			par
				//il computer sceglie in modo nondeterministico un segno
				choose $s in Sign with true do
					par
						computerChoice := $s //viene memorizzato il segno solo per vedere la giocata del computer nell'update set 
						switch(playResult(userChoice, $s))
							case WINFIRST:
								par
									outMess := "Hai vinto!"
									userWins := userWins + 1n
								endpar
							case WINSECOND:
								par
									outMess := "Ha vinto il computer."
									computerWins := computerWins + 1n
								endpar
							case PATTA:
								outMess := "Patta."
						endswitch
					endpar
				numberOfPlays := numberOfPlays + 1n
			endpar
		else
			if(userWins < computerWins) then
				outMess := "WINNER: computer"
			else
				outMess := "WINNER: user"
			endif
		endif

default init s0:
	function outMess = ""
	function userWins = 0n
	function computerWins = 0n
	function numberOfPlays = 0n
	function maxNumberOfPlays = maxNumberOfPlaysInit
